@@ -10,3 +10,10 @@ def gpkg_filepath(tmpdir):
     tmpdir_str = tmpdir.strpath
     filename = "test.gpkg"
     return os.path.join(tmpdir_str, filename)
+
+
+@pytest.fixture
+def zrd_filepath(tmpdir):
+    tmpdir_str = tmpdir.strpath
+    filename = "test.zrd"
+    return os.path.join(tmpdir_str, filename)
